<?php
/*
Plugin Name: v4 oAuth
Plugin URI: https://visual4.de/
Description: A WordPress plugin that allows users to login or register by authenticating with an existing 1CRM account via OAuth 2.0.
Version: 1.3.0
Author: visual4 GmbH
Author URI: https://visual4.de/
License: GPL2
*/

if (!defined('ABSPATH')) die;

require __DIR__ . '/v4oAuthIdentity.php';
require __DIR__ . '/v4oAuthSettings.php';
require __DIR__ . '/v4oAuthUser.php';

class v4oAuth
{
    // singleton class pattern:
    protected static $instance = NULL;

    public static function get_instance()
    {
        NULL === self::$instance and self::$instance = new self;
        return self::$instance;
    }

    public function __construct()
    {
        //settings
        new v4oAuthSettings();

        add_action('init', [$this, 'init']);
        add_action('plugins_loaded', [$this, 'localization']);
        add_filter('woocommerce_email_actions', [$this, 'woocommerce_email_actions']);
		add_action('admin_init', [$this, 'restrict_backend']);
    }

    public function init()
    {
		if (
			(is_user_logged_in() || isset($_GET['v4oauth-result']))
			&& session_status() == PHP_SESSION_NONE
		)
			session_start();

        // hook the query_vars and template_redirect so we can stay within the wordpress context no matter what (avoids having to use wp-load.php)
        add_action('template_redirect', array($this, 'qvar_handlers'));

        add_action('wp_logout', array($this, 'logout'));

        // push login messages into the DOM if the setting is enabled:
//        if (get_option('show_login_messages') !== false) {
        add_action('wp_footer', array($this, 'push_login_messages'));
        add_filter('admin_footer', array($this, 'push_login_messages'));
        add_filter('login_footer', array($this, 'push_login_messages'));
//        }
        // WooCommerce API
		// TODO check what that line was supposed to do. Function doesn't exist
		//  add_action('woocommerce_webhook_delivery', 'logout_on_delivery_failure', 10, 2);

        //scripts
        wp_enqueue_style('v4oa', plugin_dir_url(__FILE__) . 'assets/css/v4oauth.css', array());
        wp_enqueue_script('v4oa', plugin_dir_url(__FILE__) . 'assets/js/v4oauth.js', array(), false, true);
    }

	function restrict_backend()
	{
		if (
			!current_user_can('edit_posts') &&
			('/wp-admin/admin-ajax.php' != $_SERVER['PHP_SELF']) &&
			('/wp-admin/admin-post.php' != $_SERVER['PHP_SELF'])
		) {
			wp_redirect(home_url());
			exit;
		}
	}

	public function woocommerce_email_actions($email_actions)
	{
		foreach ($email_actions as $k => $email_action) {
			if($email_action == 'woocommerce_created_customer')
				unset($email_actions[$k]);
		}
		return $email_actions;
	}

	function localization()
	{
		// LOCALIZATION
		load_plugin_textdomain('v4-oauth', false, untrailingslashit(dirname(plugin_basename(__FILE__))) . '/languages');
	}

    // define the querystring variables that should trigger an action:
    function qvar_triggers()
    {
        $vars = [
            'connect',
            'code',
            'state',
            'lost_password',
            'change_password',
        ];
        return $vars;
    }

    // handle the querystring triggers:
    function qvar_handlers()
    {
        //on WooCommerce Checkout
        if (function_exists('is_checkout') && is_checkout()) {

            //TODO update user

        }

        //on query var trigger
        foreach ($this->qvar_triggers() as $qvar_trigger)
            if (isset($_GET[$qvar_trigger])) {

				if (session_status() == PHP_SESSION_NONE)
					session_start();

                do_action('before_oauth');

                try {

                    $v4oAuthIdentity = new v4oAuthIdentity();

                    switch ($qvar_trigger) {
                        case 'lost_password':

                            $v4oAuthIdentity->lost_password();

                            break;
                        case 'change_password':

                            $v4oAuthIdentity->change_password();

                            break;
                        default:

                            if ($v4oAuthIdentity->auth()) {
                                $oAuthUser = new v4oAuthUser($v4oAuthIdentity);
                                $oAuthUser->login();
                            }

                            break;
                    }

                } catch (Exception $e) {
                    $_SESSION['V4OA']['RESULT'] = __($e->getMessage(), 'v4-oauth');
					$_SESSION['V4OA']['ERROR'] = true;
                }

                do_action('after_oauth');

				$delimiter = strpos($_SESSION['V4OA']['LAST_URL'], '?') ? '&' : '?';

                header("Location: " . $_SESSION['V4OA']['LAST_URL'] . $delimiter . 'v4oauth-result');
                exit;
            }

    }

    public function logout()
    {
        $v4oAuthIdentity = new v4oAuthIdentity();
        $v4oAuthIdentity->logout();
    }

    public function push_login_messages()
    {
        if (!empty($_SESSION['V4OA']['RESULT'])) {
            $result = $_SESSION['V4OA']['RESULT'];
            unset($_SESSION['V4OA']['RESULT']);

			//is error?
			$css_class = '';
			if(isset($_SESSION['V4OA']['ERROR'])) {
				$css_class = 'error';
				unset($_SESSION['V4OA']['ERROR']);
			}

            echo "<div id='v4oauth-result' class='$css_class'>$result<span id='v4oauth-result-close'></span></div>";
        }
    }

}

define('V4OAUTHPATH', __DIR__);
v4oAuth::get_instance();