<?php

if (!defined('ABSPATH')) die;

require __DIR__ . '/vendor/autoload.php';

use \League\OAuth2\Client\Provider\GenericProvider;
use \League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Token\AccessTokenInterface;

class v4oAuthIdentity
{
    /** @var GenericProvider $provider */
    private $provider;

    private $client_id;
    private $client_secret;
    private $redirect_uri;
    private $scope;
    private $url_auth;
    private $url_token;
    private $url_resource;
    private $url_user;
    private $url_logout;
    private $url_lost_password;
    private $url_change_password;

    private $lang;

    /** @var AccessTokenInterface $access_token */
    private $access_token;
    private $me;

    /**
     * v4oAuthIdentity constructor.
     * @throws Exception
     */
    public function __construct()
    {
    error_log("oAUTHIDENTITY!!!!!!!!!!!!!!!!!!");
//        session_start();

        $_SESSION['V4OA']['PROVIDER'] = '1CRM';
        $v4oauth_option = get_option('v4_oauth');

        if(
            empty($v4oauth_option['client_id']) ||
            empty($v4oauth_option['client_secret']) ||
            empty($v4oauth_option['1crm_url'])
        )
            throw new Exception('oAuth not configured', 500);

		$locale = get_locale();
		$lang = explode('_', $locale)[0];
		$this->lang = !empty($lang) ? $lang : 'de';
		if(defined('ICL_LANGUAGE_CODE'))
			$this->lang = ICL_LANGUAGE_CODE;

        $this->client_id = $v4oauth_option['client_id'];
        $this->client_secret = $v4oauth_option['client_secret'];

        $this->redirect_uri = rtrim(site_url(), '/') . '/';

        $this->scope = 'basic'; // PROVIDER SPECIFIC: 'profile' is the minimum scope required to get the user's id from 1crm

		$login_server_url = trailingslashit($v4oauth_option['1crm_url']);
		if(empty($v4oauth_option['disable_auto_url_suffix']) || !$v4oauth_option['disable_auto_url_suffix'])
			$login_server_url .= "loginServer/index.php/";

        $this->url_auth = $login_server_url . "authorize";
        $this->url_token = $login_server_url . "access_token";
        $this->url_resource = $login_server_url . "resource";
        $this->url_user = $login_server_url . "api/me";
        $this->url_logout = $login_server_url . "logout";
        $this->url_lost_password = $login_server_url . "lostpassword";
        $this->url_change_password = $login_server_url . "changepassword";

        $this->provider = new GenericProvider([
            'clientId' => $this->client_id,    // The client ID assigned to you by the provider
            'clientSecret' => $this->client_secret,   // The client password assigned to you by the provider
            'redirectUri' => $this->redirect_uri,
            'urlAuthorize' => $this->url_auth,
            'urlAccessToken' => $this->url_token,
            'urlResourceOwnerDetails' => $this->url_resource,
            'scopes' => [$this->scope],
			'pkceMethod' => GenericProvider::PKCE_METHOD_S256,
//            'scopeSeparator' => ',',
        ]);
    }

    private function create_redirect_url()
    {
        /*
         * redirect URL
         */
        // try to obtain the redirect_url from the default login page:
        $redirect_url = !empty($_GET['redirect_to']) ? esc_url($_GET['redirect_to']) : false;

        // if no redirect_url was found, set it to the user's last page:
        if (!$redirect_url) {
//            $redirect_url = explode('?', (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]", "?")[0];
            $redirect_url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
            $redirect_url = explode('?', $redirect_url);
            $redirect_url = $redirect_url[0];
        }

        //if still false get_home_url
        if (!$redirect_url) {
            $redirect_url = explode('?', function_exists('wpml_get_home_url_filter') ? wpml_get_home_url_filter() : site_url())[0];
        }

        return $redirect_url;
    }

    /**
     * @throws Exception
     */
    public function auth()
    {
        /*
         * handle already logged in users
         */
        if (is_user_logged_in() && $_SESSION['V4OA']['EXPIRES'] > time()) {
            //don't process
            return false;
        }

        /*
         * Process
         */
        //If we don't have an authorization code then get one
        if (!isset($_GET['code'])) {

            $redirect_url = $this->create_redirect_url();

            // set the user's last page, so we can return that user there after they log in:
            $_SESSION['V4OA']['LAST_URL'] = $redirect_url;
            $_SESSION['V4OA']['LANG'] = $this->lang;


            /*
             * auth
             */
            // Fetch the authorization URL from the provider; this returns the
            // urlAuthorize option and generates and applies any necessary parameters
            // (e.g. state).
            $authorizationUrl = $this->provider->getAuthorizationUrl();
            $authorizationUrl .= '&lang=' . ($this->lang);
            
            //providers
            $provider = $_GET['connect'];
            if(in_array($provider, array_keys(v4oAuthSettings::$providers)))
                $authorizationUrl .= '&provider=' . $provider;

            // Get the state generated for you and store it to the session.
            $_SESSION['V4OA']['OAUTH2STATE'] = $this->provider->getState();
			$_SESSION['V4OA']['OAUTH2PKCECODE'] = $this->provider->getPkceCode();

            // Redirect the user to the authorization URL.
            header('Location: ' . $authorizationUrl);
            exit;

            // Check given state against previously stored one to mitigate CSRF attack
        } elseif (empty($_GET['state']) || (isset($_SESSION['V4OA']['OAUTH2STATE']) && $_GET['state'] !== $_SESSION['V4OA']['OAUTH2STATE'])) {

            if (isset($_SESSION['V4OA']['OAUTH2STATE'])) {
                unset($_SESSION['V4OA']['OAUTH2STATE']);
            }

            $this->error('Invalid state');

        } else {

            try {

				// Optional, only required when PKCE is enabled.
				// Restore the PKCE code stored in the session.
				$this->provider->setPkceCode($_SESSION['V4OA']['OAUTH2PKCECODE']);

                // Try to get an access token using the authorization code grant.
                $accessToken = $this->provider->getAccessToken('authorization_code', [
                    'code' => $_GET['code']
                ]);
                // We have an access token, which we may use in authenticated

                if ($accessToken->hasExpired()) {
                    $accessToken = $this->provider->getAccessToken('refresh_token', [
                        'refresh_token' => $accessToken->getRefreshToken()
                    ]);
                }

                $_SESSION['V4OA']['ACCESS_TOKEN'] = $accessToken->getToken();
                $_SESSION['V4OA']['EXPIRES'] = $accessToken->getExpires();

                $this->access_token = $accessToken;

                return true;

            } catch (IdentityProviderException $e) {

            	$response_body = $e->getResponseBody();

				error_log('PHP Error: v4OAuthIdentity (' . $e->getCode() . ') ' . json_encode($response_body));
                // Failed to get the access token or user details.
                $this->error($e->getMessage());

            }

        }

        exit('oops, that should not have happened. please try again, or contact the administrator (auth error)');

    }

    public function lost_password()
    {
		$redirect_url = $this->create_redirect_url();

		// set the user's last page so, we can return that user there after they log in:
		$_SESSION['V4OA']['LAST_URL'] = $redirect_url;
		$_SESSION['V4OA']['LANG'] = $this->lang;

		//lost password
		if (isset($_GET['lost_password'])) {
			// Redirect the user to the password change URL.
			header('Location: ' . $this->url_lost_password . '?redirect_uri=' . $redirect_url);
			exit;
		}
    }

    public function change_password()
    {
        if (isset($_GET['result'])) {
			if(!empty($_GET['result'])){
				$result_str = 'Password changed';
			} else {
				$result_str = 'Password change failed';
				$_SESSION['V4OA']['ERROR'] = true;
			}
			$_SESSION['V4OA']['RESULT'] = __( $result_str, 'v4-oauth');
        } else {
            $redirect_url = $this->create_redirect_url();

            // set the user's last page so we can return that user there after they login:
            $_SESSION['V4OA']['LAST_URL'] = $redirect_url;
            $_SESSION['V4OA']['LANG'] = $this->lang;

            //change password
            if (isset($_GET['change_password'])) {
                // Redirect the user to the password change URL.
                header('Location: ' . $this->url_change_password . '?redirect_uri=' . $redirect_url);
                exit;
            }
        }
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function api_me()
    {
        try {
            // The provider provides a way to get an authenticated API request for
            // the service, using the access token; it returns an object conforming
            // to Psr\Http\Message\RequestInterface.
            $params = array(
                //'access_token' => $_SESSION['V4OA']['ACCESS_TOKEN'], // PROVIDER SPECIFIC: the access_token is passed to 1crm via POST param
                'lang' => $this->lang
            );
            $url_params = http_build_query($params);
            $url = $this->url_user . '?' . $url_params . '&XDEBUG_SESSION_START=PHPSTORM';

            $request = $this->provider->getAuthenticatedRequest(
                'GET',
                $url,
                $this->access_token
            );

            //return parsed response
            $this->me = $this->provider->getParsedResponse($request);
            return $this->me;

        } catch (IdentityProviderException $e) {

			$response_body = $e->getResponseBody();
			error_log('PHP Error: v4OAuthIdentity (' . $e->getCode() . ') ' . json_encode($response_body));
            // Failed to get the access token or user details.
            $this->error($e->getMessage());

        }

        exit('oops, that should not have happened. please try again, or contact the administrator (request_api_me error)');
    }

    public function logout()
    {
        # AUTHENTICATION FLOW #
        // the oauth 2.0 authentication flow will start in this script and make several calls to the third-party authentication provider which in turn will make callbacks to this script that we continue to handle until the login completes with a success or failure:

        // try to obtain the redirect_url from the default login page:
        $redirect_url = !empty($_GET['redirect_to']) ? esc_url($_GET['redirect_to']) : '';
        // if no redirect_url was found, set it to the user's last page:
        if (!$redirect_url) {
            $redirect_url = strtok($_SERVER['HTTP_REFERER'], "?");
        }
        // set the user's last page, so we can return that user there after they log in:
        unset($_SESSION['V4OA']['LAST_URL']);

        $params = array(
            'redirect_uri' => $redirect_url
        );
        $url_params = http_build_query($params);
        $url = $this->url_logout . '?' . $url_params;
        header('Location: ' . $url, true, 301);
        die();

    }

    /**
     * @param $msg
     * @param int $code
     * @throws Exception
     */
    private function error($msg, $code = 400)
    {
        throw new Exception($msg, $code);
    }

}